@@ -67,6 +67,10 @@ const struct valstr ipmi_oem_info[] = {
    { IPMI_OEM_NOKIA_SIEMENS_NETWORKS, "Nokia Siemens Networks" },
    { IPMI_OEM_BULL,                   "Bull Company" },
    { IPMI_OEM_PPS,                    "Pigeon Point Systems" },
+   { IPMI_OEM_BROADCOM,               "Broadcom Corporation" },
+   { IPMI_OEM_ERICSSON,               "Ericsson AB"},
+   { IPMI_OEM_QUANTA,                 "Quanta" },
+   { IPMI_OEM_ADVANTECH,              "Advantech" },
    { 0xffff , NULL },
 };
 
@@ -94,8 +98,10 @@ const struct oemvalstr ipmi_oem_product_info[] = {
    { IPMI_OEM_KONTRON,5504, "AM4300 AdvancedMC" },
    { IPMI_OEM_KONTRON,5507, "AM4301 AdvancedMC" },
    { IPMI_OEM_KONTRON,5508, "AM4330 AdvancedMC" },
+   { IPMI_OEM_KONTRON,5520, "KTC5520/EATX" },
    { IPMI_OEM_KONTRON,5703, "RTM8020" },
    { IPMI_OEM_KONTRON,5704, "RTM8030" },
+   { IPMI_OEM_KONTRON,5705, "RTM8050" },
    { IPMI_OEM_KONTRON,6000, "CP6000" },
    { IPMI_OEM_KONTRON,6006, "DT-64" },
    { IPMI_OEM_KONTRON,6010, "CP6010" },
@@ -107,10 +113,97 @@ const struct oemvalstr ipmi_oem_product_info[] = {
    { IPMI_OEM_KONTRON,5004, "AT8020" },
    { IPMI_OEM_KONTRON,5006, "AT8030 IPMC" },
    { IPMI_OEM_KONTRON,2025, "AT8030 MMC" },
+   { IPMI_OEM_KONTRON,5007, "AT8050" },
    { IPMI_OEM_KONTRON,5301, "AT8400" },
    { IPMI_OEM_KONTRON,5303, "AT8901" },
+   /* Broadcom */
+   { IPMI_OEM_BROADCOM, 5725, "BCM5725" },
+   /* Ericsson */
+   { IPMI_OEM_ERICSSON, 0x0054, "Phantom" },
+   /* Advantech */
+   /* ATCA Blades */
+   { IPMI_OEM_ADVANTECH, 0x3393, "MIC-3393" },
+   { IPMI_OEM_ADVANTECH, 0x3395, "MIC-3395" },
+   { IPMI_OEM_ADVANTECH, 0x3396, "MIC-3396" },
+   { IPMI_OEM_ADVANTECH, 0x5302, "MIC-5302" },
+   { IPMI_OEM_ADVANTECH, 0x5304, "MIC-5304" },
+   { IPMI_OEM_ADVANTECH, 0x5320, "MIC-5320" },
+   { IPMI_OEM_ADVANTECH, 0x5321, "MIC-5321" },
+   { IPMI_OEM_ADVANTECH, 0x5322, "MIC-5322" },
+   { IPMI_OEM_ADVANTECH, 0x5332, "MIC-5332" },
+   { IPMI_OEM_ADVANTECH, 0x5333, "MIC-5333" },
+   { IPMI_OEM_ADVANTECH, 0x5342, "MIC-5342" },
+   { IPMI_OEM_ADVANTECH, 0x5343, "MIC-5343" },
+   { IPMI_OEM_ADVANTECH, 0x5344, "MIC-5344" },
+   { IPMI_OEM_ADVANTECH, 0x5345, "MIC-5345" },
+   { IPMI_OEM_ADVANTECH, 0x5201, "MIC-5201 Dual 10GE AMC"},
+   { IPMI_OEM_ADVANTECH, 0x5203, "MIC-5203 Quad GbE AMC"},
+   { IPMI_OEM_ADVANTECH, 0x5212, "MIC-5212 Dual 10GE AMC"},
+   /* AdvancedMC */
+   { IPMI_OEM_ADVANTECH, 0x5401, "MIC-5401" },
+   { IPMI_OEM_ADVANTECH, 0x5601, "MIC-5601" },
+   { IPMI_OEM_ADVANTECH, 0x5602, "MIC-5602" },
+   { IPMI_OEM_ADVANTECH, 0x5604, "MIC-5604" },
+   { IPMI_OEM_ADVANTECH, 0x5603, "MIC-5603" },
+   { IPMI_OEM_ADVANTECH, 0x6311, "MIC-6311" },
+   { IPMI_OEM_ADVANTECH, 0x6313, "MIC-6313" },
+   { IPMI_OEM_ADVANTECH, 0x8301, "MIC-8301" },
+   { IPMI_OEM_ADVANTECH, 0x8302, "MIC-8302" },
+   { IPMI_OEM_ADVANTECH, 0x8304, "MIC-8304" },
+   { IPMI_OEM_ADVANTECH, 0x5101, "RTM-5101" },
+   { IPMI_OEM_ADVANTECH, 0x5102, "RTM-5102" },
+   { IPMI_OEM_ADVANTECH, 0x5106, "RTM-5106" },
+   { IPMI_OEM_ADVANTECH, 0x5107, "RTM-5107" },
+   { IPMI_OEM_ADVANTECH, 0x5210, "RTM-5210" },
+   { IPMI_OEM_ADVANTECH, 0x5220, "RTM-5220" },
+   { IPMI_OEM_ADVANTECH, 0x5104, "RTM-5104" },
+   { IPMI_OEM_ADVANTECH, 0x5500, "UTCA-5500"},
+   { IPMI_OEM_ADVANTECH, 0x5503, "UTCA-5503"},
+   { IPMI_OEM_ADVANTECH, 0x5504, "UTCA-5504"},
+   { IPMI_OEM_ADVANTECH, 0x5801, "UTCA-5801"},
+   { IPMI_OEM_ADVANTECH, 0x2210, "NCPB-2210"},
+   { IPMI_OEM_ADVANTECH, 0x2305, "NCPB-2305"},
+   { IPMI_OEM_ADVANTECH, 0x2320, "NCPB-2320"},
+   { IPMI_OEM_ADVANTECH, 0x3109, "NCP-3109" },
+   { IPMI_OEM_ADVANTECH, 0x3110, "NCP-3110" },
+   { IPMI_OEM_ADVANTECH, 0x3200, "NCP-3200" },
+   { IPMI_OEM_ADVANTECH, 0x5060, "SMM-5060" },
+   { IPMI_OEM_ADVANTECH, 0x3210, "FWA-3210" },
+   { IPMI_OEM_ADVANTECH, 0x3220, "FWA-3220" },
+   { IPMI_OEM_ADVANTECH, 0x3221, "FWA-3221" },
+   { IPMI_OEM_ADVANTECH, 0x3230, "FWA-3230" },
+   { IPMI_OEM_ADVANTECH, 0x3231, "FWA-3231" },
+   { IPMI_OEM_ADVANTECH, 0x3233, "FWA-3233" },
+   { IPMI_OEM_ADVANTECH, 0x3250, "FWA-3250" },
+   { IPMI_OEM_ADVANTECH, 0x3260, "FWA-3260" },
+   { IPMI_OEM_ADVANTECH, 0x5020, "FWA-5020" },
+   { IPMI_OEM_ADVANTECH, 0x6510, "FWA-6510" },
+   { IPMI_OEM_ADVANTECH, 0x6511, "FWA-6511" },
+   { IPMI_OEM_ADVANTECH, 0x6512, "FWA-6512" },
+   { IPMI_OEM_ADVANTECH, 0x6520, "FWA-6520" },
+   { IPMI_OEM_ADVANTECH, 0x6521, "FWA-6521" },
+   { IPMI_OEM_ADVANTECH, 0x6522, "FWA-6522" },
+   { IPMI_OEM_ADVANTECH, 0x7310, "ATCA-7310"},
+   { IPMI_OEM_ADVANTECH, 0x7330, "ATCA-7330"},
+   { IPMI_OEM_ADVANTECH, 0x7410, "ATCA-7410"},
+   { IPMI_OEM_ADVANTECH, 0x9023, "ATCA-9023"},
+   { IPMI_OEM_ADVANTECH, 0x9112, "ATCA-9112"},
+   { IPMI_OEM_ADVANTECH, 0x4201, "AMC-4201" },
+   { IPMI_OEM_ADVANTECH, 0x4202, "AMC-4202" },
+   { IPMI_OEM_ADVANTECH, 0x3211, "NAMB-3211"},
+   { IPMI_OEM_ADVANTECH, 0x1207, "CPCI-1207"},
+   { IPMI_OEM_ADVANTECH, 0x120E, "CPCI-1207 Test Board"},
+   { IPMI_OEM_ADVANTECH, 0x1304, "CPCI-1304"},
+   { IPMI_OEM_ADVANTECH, 0x7001, "CPCI-7001"},
+   { IPMI_OEM_ADVANTECH, 0x8220, "CPCI-8220"},
+   { IPMI_OEM_ADVANTECH, 0x9001, "ESP-9001" },
+   { IPMI_OEM_ADVANTECH, 0x9002, "ESP-9002" },
+   { IPMI_OEM_ADVANTECH, 0x9012, "ESP-9012" },
+   { IPMI_OEM_ADVANTECH, 0x9212, "ESP-9212" },
+   { IPMI_OEM_ADVANTECH, 0x6000, "CGS-6000" },
+   { IPMI_OEM_ADVANTECH, 0x6010, "CGS-6010" },
 
-   { 0xffff        , 0xffff , NULL },
+   { 0xffffff        , 0xffff , NULL },
  };
 
 const struct oemvalstr ipmi_oem_sdr_type_vals[] = {
@@ -123,11 +216,24 @@ const struct oemvalstr ipmi_oem_sdr_type_vals[] = {
    { IPMI_OEM_KONTRON , 0xC6 , "OEM POST Value Sensor" },
    { IPMI_OEM_KONTRON , 0xC7 , "OEM FWUM Status" },
    { IPMI_OEM_KONTRON , 0xC8 , "OEM Switch Mngt Software Status" },
+   { IPMI_OEM_KONTRON , 0xC9 , "OEM OEM Diagnostic Status" },
+   { IPMI_OEM_KONTRON , 0xCA , "OEM Component Firmware Upgrade" },
+   { IPMI_OEM_KONTRON , 0xCB , "OEM FRU Over Current" },
+   { IPMI_OEM_KONTRON , 0xCC , "OEM FRU Sensor Error" },
+   { IPMI_OEM_KONTRON , 0xCD , "OEM FRU Power Denied" },
+   { IPMI_OEM_KONTRON , 0xCE , "OEM Reserved" },
+   { IPMI_OEM_KONTRON , 0xCF , "OEM Board Reset" },
+   { IPMI_OEM_KONTRON , 0xD0 , "OEM Clock Resource Control" },
+   { IPMI_OEM_KONTRON , 0xD1 , "OEM Power State" },
+   { IPMI_OEM_KONTRON , 0xD2 , "OEM FRU Mngt Power Failure" },
+   { IPMI_OEM_KONTRON , 0xD3 , "OEM Jumper Status" },
+   { IPMI_OEM_KONTRON , 0xF2 , "OEM RTM Module Hotswap" },
 
    { IPMI_OEM_PICMG   , 0xF0 , "PICMG FRU Hotswap" },
    { IPMI_OEM_PICMG   , 0xF1 , "PICMG IPMB0 Link State" },
+   { IPMI_OEM_PICMG   , 0xF2 , "PICMG Module Hotswap" },
 
-   { 0xffff,            0x00,  NULL }
+   { 0xffffff,            0x00,  NULL }
 };
 
 const struct valstr ipmi_netfn_vals[] = {
@@ -188,6 +294,7 @@ const struct valstr ipmi_authtype_session_vals[] = {
 	{ IPMI_SESSION_AUTHTYPE_MD5,      "MD5" },
 	{ IPMI_SESSION_AUTHTYPE_PASSWORD, "PASSWORD" },
 	{ IPMI_SESSION_AUTHTYPE_OEM,      "OEM" },
+	{ IPMI_SESSION_AUTHTYPE_RMCP_PLUS,"RMCP+" },
 	{ 0xFF,                           NULL },
 };
 
@@ -243,26 +350,40 @@ const struct valstr entity_id_vals[] = {
 	{ 0x26, "Remote Management Device" },
 	{ 0x27, "External Environment" },
 	{ 0x28, "Battery" },
-   { 0x29, "Processing Blade" },
-   { 0x2A, "Connectivity Switch" },
-   { 0x2B, "Processor/Memory Module" },
-   { 0x2C, "I/O Module" },
-   { 0x2D, "Processor/IO Module" },
-   { 0x2E, "Management Controller Firmware" },
-   { 0x2F, "IPMI Channel" },
-   { 0x30, "PCI Bus" },
-   { 0x31, "PCI Express Bus" },
-   { 0x32, "SCSI Bus (parallel)" },
-   { 0x33, "SATA/SAS Bus" },
-   { 0x34, "Processor/Front-Side Bus" },
-   /* PICMG */
-   { 0xA0, "PICMG Front Board" },
-   { 0xC0, "PICMG Rear Transition Module" },
-   { 0xC1, "PICMG AdvancedMC Module" },
-   { 0xF0, "PICMG Shelf Management Controller" },
-   { 0xF1, "PICMG Filtration Unit" },
-   { 0xF2, "PICMG Shelf FRU Information" },
-   { 0xF3, "PICMG Alarm Panel" },
+	{ 0x29, "Processing Blade" },
+	{ 0x2A, "Connectivity Switch" },
+	{ 0x2B, "Processor/Memory Module" },
+	{ 0x2C, "I/O Module" },
+	{ 0x2D, "Processor/IO Module" },
+	{ 0x2E, "Management Controller Firmware" },
+	{ 0x2F, "IPMI Channel" },
+	{ 0x30, "PCI Bus" },
+	{ 0x31, "PCI Express Bus" },
+	{ 0x32, "SCSI Bus (parallel)" },
+	{ 0x33, "SATA/SAS Bus" },
+	{ 0x34, "Processor/Front-Side Bus" },
+	{ 0x35, "Real Time Clock(RTC)" },
+	{ 0x36, "Reserved" },
+	{ 0x37, "Air Inlet" },
+	{ 0x38, "Reserved" },
+	{ 0x39, "Reserved" },
+	{ 0x3A, "Reserved" },
+	{ 0x3B, "Reserved" },
+	{ 0x3C, "Reserved" },
+	{ 0x3D, "Reserved" },
+	{ 0x3E, "Reserved" },
+	{ 0x3F, "Reserved" },
+	{ 0x40, "Air Inlet" },
+	{ 0x41, "Processor" },
+	{ 0x42, "Baseboard/Main System Board" },
+	/* PICMG */
+	{ 0xA0, "PICMG Front Board" },
+	{ 0xC0, "PICMG Rear Transition Module" },
+	{ 0xC1, "PICMG AdvancedMC Module" },
+	{ 0xF0, "PICMG Shelf Management Controller" },
+	{ 0xF1, "PICMG Filtration Unit" },
+	{ 0xF2, "PICMG Shelf FRU Information" },
+	{ 0xF3, "PICMG Alarm Panel" },
 	{ 0x00, NULL },
 };
 
@@ -371,6 +492,7 @@ const struct valstr completion_code_vals[] = {
 	{ 0xd3, "Destination unavailable" },
 	{ 0xd4, "Insufficient privilege level" },
 	{ 0xd5, "Command not supported in present state" },
+	{ 0xd6, "Cannot execute command, command disabled" },
 	{ 0xff, "Unspecified error" },
 	{ 0x00, NULL }
 };
@@ -407,3 +529,137 @@ const struct valstr ipmi_encryption_algorithms[] = {
 	{ IPMI_CRYPT_XRC4_40,     "xrc4_40"     },
 	{ 0x00, NULL }
 };
+
+const struct valstr picmg_frucontrol_vals[] = {
+	{ 0, "Cold Reset" },
+	{ 1, "Warm Reset"  },
+	{ 2, "Graceful Reboot" },
+	{ 3, "Issue Diagnostic Interrupt" },
+	{ 4, "Quiesce" },
+	{ 5, NULL },
+};
+
+const struct valstr picmg_clk_family_vals[] = {
+	{ 0x00, "Unspecified" },
+	{ 0x01, "SONET/SDH/PDH" },
+	{ 0x02, "Reserved for PCI Express" },
+	{ 0x03, "Reserved" }, /* from 03h to C8h */
+	{ 0xC9, "Vendor defined clock family" }, /* from C9h to FFh */
+	{ 0x00, NULL },
+};
+
+const struct oemvalstr picmg_clk_accuracy_vals[] = {
+	{ 0x01, 10, "PRS" },
+	{ 0x01, 20, "STU" },
+	{ 0x01, 30, "ST2" },
+	{ 0x01, 40, "TNC" },
+	{ 0x01, 50, "ST3E" },
+	{ 0x01, 60, "ST3" },
+	{ 0x01, 70, "SMC" },
+	{ 0x01, 80, "ST4" },
+	{ 0x01, 90, "DUS" },
+   { 0x02, 0xE0, "PCI Express Generation 2" },
+   { 0x02, 0xF0, "PCI Express Generation 1" },
+   { 0xffffff, 0x00,  NULL }
+};
+
+const struct oemvalstr picmg_clk_resource_vals[] = {
+	{ 0x0, 0, "On-Carrier Device 0" },
+	{ 0x0, 1, "On-Carrier Device 1" },
+	{ 0x1, 1, "AMC Site 1 - A1" },
+	{ 0x1, 2, "AMC Site 1 - A2" },
+	{ 0x1, 3, "AMC Site 1 - A3" },
+	{ 0x1, 4, "AMC Site 1 - A4" },
+	{ 0x1, 5, "AMC Site 1 - B1" },
+	{ 0x1, 6, "AMC Site 1 - B2" },
+	{ 0x1, 7, "AMC Site 1 - B3" },
+	{ 0x1, 8, "AMC Site 1 - B4" },
+   { 0x2, 0, "ATCA Backplane" },
+   { 0xffffff, 0x00,  NULL }
+};
+
+const struct oemvalstr picmg_clk_id_vals[] = {
+	{ 0x0, 0, "Clock 0" },
+	{ 0x0, 1, "Clock 1" },
+	{ 0x0, 2, "Clock 2" },
+	{ 0x0, 3, "Clock 3" },
+	{ 0x0, 4, "Clock 4" },
+	{ 0x0, 5, "Clock 5" },
+	{ 0x0, 6, "Clock 6" },
+	{ 0x0, 7, "Clock 7" },
+	{ 0x0, 8, "Clock 8" },
+	{ 0x0, 9, "Clock 9" },
+	{ 0x0, 10, "Clock 10" },
+	{ 0x0, 11, "Clock 11" },
+	{ 0x0, 12, "Clock 12" },
+	{ 0x0, 13, "Clock 13" },
+	{ 0x0, 14, "Clock 14" },
+	{ 0x0, 15, "Clock 15" },
+	{ 0x1, 1, "TCLKA" },
+	{ 0x1, 2, "TCLKB" },
+	{ 0x1, 3, "TCLKC" },
+	{ 0x1, 4, "TCLKD" },
+	{ 0x1, 5, "FLCKA" },
+   { 0x2, 1, "CLK1A" },
+   { 0x2, 2, "CLK1A" },
+   { 0x2, 3, "CLK1A" },
+   { 0x2, 4, "CLK1A" },
+   { 0x2, 5, "CLK1A" },
+   { 0x2, 6, "CLK1A" },
+   { 0x2, 7, "CLK1A" },
+   { 0x2, 8, "CLK1A" },
+   { 0x2, 9, "CLK1A" },
+   { 0xffffff, 0x00,  NULL }
+};
+
+const struct valstr picmg_busres_id_vals[] = {
+   { 0x0, "Metallic Test Bus pair #1" },
+   { 0x1, "Metallic Test Bus pair #2" },
+   { 0x2, "Synch clock group 1 (CLK1)" },
+   { 0x3, "Synch clock group 2 (CLK2)" },
+   { 0x4, "Synch clock group 3 (CLK3)" },
+	{ 0x5, NULL }
+};
+const struct valstr picmg_busres_board_cmd_vals[] = {
+  { 0x0, "Query" },
+  { 0x1, "Release" },
+  { 0x2, "Force" },
+  { 0x3, "Bus Free" },
+  { 0x4, NULL }
+};
+
+const struct valstr picmg_busres_shmc_cmd_vals[] = {
+  { 0x0, "Request" },
+  { 0x1, "Relinquish" },
+  { 0x2, "Notify" },
+  { 0x3, NULL }
+};
+
+const struct oemvalstr picmg_busres_board_status_vals[] = {
+ { 0x0, 0x0, "In control" },
+ { 0x0, 0x1, "No control" },
+ { 0x1, 0x0, "Ack" },
+ { 0x1, 0x1, "Refused" },
+ { 0x1, 0x2, "No control" },
+ { 0x2, 0x0, "Ack" },
+ { 0x2, 0x1, "No control" },
+ { 0x3, 0x0, "Accept" },
+ { 0x3, 0x1, "Not Needed" },
+ { 0xffffff, 0x00,  NULL }
+};
+
+const struct oemvalstr picmg_busres_shmc_status_vals[] = {
+ { 0x0, 0x0, "Grant" },
+ { 0x0, 0x1, "Busy" },
+ { 0x0, 0x2, "Defer" },
+ { 0x0, 0x3, "Deny" },
+
+ { 0x1, 0x0, "Ack" },
+ { 0x1, 0x1, "Error" },
+
+ { 0x2, 0x0, "Ack" },
+ { 0x2, 0x1, "Error" },
+ { 0x2, 0x2, "Deny" },
+
+ { 0xffffff, 0x00,  NULL }
+};
